## -*- texinfo -*-
## @deftypefn {Function File} {} SvgPlotDraw (@var{svgPlot}, @var{plotData}, @var{x}, @var{y}, @var{width}, @var{height})
## Make a new plot of @var{plotData} in (@var{x},@var{y}) coordinates and size (@var{width}, @var{height}).
## @seealso{SvgPlotNew}
## @seealso{svgcanvasload, svgcanvassave, svgcanvas_delete_on_load}
## @end deftypefn

function [svgPlot]=SvgPlotDraw(svgPlot, plotData, x, y, width, height)
	out=svgPlot.out;
	
	%Set definitions of points
	%fprintf(out, '<defs>\n');
	%for n=1:size(plotData{1}.points)(1)
		%point=plotData{1}.points(n,:);
		%fprintf(out, point, n );
	%endfor
	%fprintf(out, '</defs>\n');
	
	eval('x;', 'x=0; y=0; width=svgPlot.width; height=svgPlot.height; ');
	
	border_left=50;
	border_right=20;
	border_botton=40;
	border_top=10;
	
	fprintf(out, '<rect x="%d" y="%d" width="%d" height="%d" fill="white" stroke="black" stroke-width="1" />\n', x, y, width-1, height-1);
	
	xmin=plotData{1}.xMin;
	xmax=plotData{1}.xMax;
	ymin=plotData{1}.yMin;
	ymax=plotData{1}.yMax;
	numbersfontsize=plotData{1}.numbersFontSize;
	numbersfont=plotData{1}.numbersFont;
	grid=plotData{1}.grid;
	
	%Autoscale
	xscale=ScaleLinear(xmin, xmax);
	yscale=ScaleLinear(ymin, ymax);
	
	% Plots tics and grid

	fprintf(out, '<rect x="%d" y="%d" width="%d" height="%d" fill="white" stroke="black" stroke-width="2" />\n', x+border_left, y+border_top, width-border_left-border_right, height-border_botton-border_top);
	
	for ny=yscale
		yy=y+border_top+(height-border_botton-border_top)-(ny-ymin)*(height-border_botton-border_top)/(ymax-ymin);
		if(ny<ymin || ny>ymax)
			continue;
		endif;
			
		fprintf(out, '<g transform="translate(%f,%f)" style="stroke:none; fill:black; font-family:%s; font-size:%fpt; text-anchor:end">\n', x+border_left-8, yy+numbersfontsize/2, numbersfont, numbersfontsize);
         	fprintf(out,        '<text> %6.2g </text>\n', ny);
        	fprintf(out, '</g>\n');
        	fprintf(out, '<line x1="%f" y1="%f" x2="%f" y2="%f" stroke="black" stroke-width="2" />\n', x+border_left-4, yy, x+border_left, yy );
		if(grid)
			fprintf(out, '<line x1="%f" y1="%f" x2="%f" y2="%f" stroke="gray" stroke-width="0.5" />\n', x+border_left+1, yy, x+width - border_right, yy );
		endif
        endfor
        
        for nx=xscale
		xx=x+border_left+(nx-xmin)*(width-border_left-border_right)/(xmax-xmin);
		
		if(nx<xmin || nx>xmax)
			continue;
		endif;
		
		fprintf(out, '<g transform="translate(%f,%f)" style="stroke:none; fill:black; font-family:%s; font-size:%fpt; text-anchor:middle">\n', xx, y+height-border_botton+numbersfontsize+3, numbersfont, numbersfontsize);
         	fprintf(out,        '<text> %6g </text>\n', nx);
        	fprintf(out, '</g>\n');
        	fprintf(out, '<line x1="%f" y1="%f" x2="%f" y2="%f" stroke="black" stroke-width="2" />\n', xx, y+height-border_botton+4, xx, y+height-border_botton);
		if(grid)
			fprintf(out, '<line x1="%f" y1="%f" x2="%f" y2="%f" stroke="gray" stroke-width="0.5" />\n', xx, y+height-border_botton-1, xx, y+border_top+1);
		endif
        endfor
        
        %Plots labels
        
        if(length(plotData{1}.xLabel)>0)
        	xx=x+(border_left+width-border_right)/2;
        	yy=y+height-border_botton+numbersfontsize+plotData{1}.fontLabelSize+2;
	fprintf(out, '<g transform="translate(%f,%f)" style="stroke:none; fill:black; font-family:%s; font-size:%fpt; text-anchor:middle">\n', xx, yy,plotData{1}.fontLabel, plotData{1}.fontLabelSize);
         	fprintf(out,        '<text>%s</text>\n', plotData{1}.xLabel);
        	fprintf(out, '</g>\n');
        endif
         if(length(plotData{1}.yLabel)>0)
        	xx=x+border_left+plotData{1}.fontLabelSize-50;
        	yy=y+(height-border_botton+border_top)/2;
		fprintf(out, '<g transform="translate(%f,%f) rotate(-90)" style="stroke:none; fill:black; font-family:%s; font-size:%fpt; text-anchor:middle">\n', xx, yy,plotData{1}.fontLabel, plotData{1}.fontLabelSize);
         	fprintf(out,        '<text>%s</text>\n', plotData{1}.yLabel);
        	fprintf(out, '</g>\n');
        endif

	% Plots function
	
	for n=2:length(plotData)
		if(plotData{n}.line_style!=0)
		% Plots lines
		
		fprintf(out, '<path d="');
		nx=1;
		if ( plotData{n}.x(nx)<xmin || plotData{n}.x(nx)>xmax || plotData{n}.y(nx)<ymin || plotData{n}.y(nx)>ymax)
				in_ok=0;
		else
				in_ok=1;
				xx=x+border_left+(plotData{n}.x(nx)-xmin)*(width-border_left-border_right)/(xmax-xmin);
				yy=y+border_top+(height-border_botton-border_top)-(plotData{n}.y(nx)-ymin)*(height-border_botton-border_top)/(ymax-ymin);
				fprintf(out,"M %f %f ", xx, yy );
		endif
		
		for nx=2:(length(plotData{n}.x))
			
			x0=plotData{n}.x(nx-1); y0=plotData{n}.y(nx-1);
			x1=plotData{n}.x(nx); y1=plotData{n}.y(nx);
			if(in_ok) % Last point inside plot margins
				if ( x1<xmin || x1>xmax || y1<ymin || y1>ymax)
					in_ok=0; % One point inside and the other outside
					[x1, y1, ok]=intersec1(x0, y0, x1, y1, xmin, ymin, xmax, ymax);
				else
					in_ok=1; % Two points are inside
				endif
				xx=x+border_left+(x1-xmin)*(width-border_left-border_right)/(xmax-xmin);
				yy=y+border_top+(height-border_botton-border_top)-(y1-ymin)*(height-border_botton-border_top)/(ymax-ymin);
				fprintf(out,"L %f %f ", xx, yy );
			else
				if ( x1<xmin || x1>xmax || y1<ymin || y1>ymax)
					in_ok=0; % Both outside
					[x0,y0,x1,y1, ok]=intersec2(x0, y0, x1, y1, xmin, ymin, xmax, ymax);
					if(ok) % Checks if line cuts plots margins
						xx0=x+border_left+(x0-xmin)*(width-border_left-border_right)/(xmax-xmin);
						yy0=y+border_top+(height-border_botton-border_top)-(y0-ymin)*(height-border_botton-border_top)/(ymax-ymin);
						xx=x+border_left+(x1-xmin)*(width-border_left-border_right)/(xmax-xmin);
						yy=y+border_top+(height-border_botton-border_top)-(y1-ymin)*(height-border_botton-border_top)/(ymax-ymin);
						fprintf(out, 'M %f %f L %f %f ', xx0, yy0, xx, yy);
					endif
				else
					in_ok=1; % One point inside and the other outside
					[x0, y0, ok]=intersec1(x0, y0, x1, y1, xmin, ymin, xmax, ymax);
					xx0=x+border_left+(x0-xmin)*(width-border_left-border_right)/(xmax-xmin);
					yy0=y+border_top+(height-border_botton-border_top)-(y0-ymin)*(height-border_botton-border_top)/(ymax-ymin);
					xx=x+border_left+(x1-xmin)*(width-border_left-border_right)/(xmax-xmin);
					yy=y+border_top+(height-border_botton-border_top)-(y1-ymin)*(height-border_botton-border_top)/(ymax-ymin);
					fprintf(out, 'M %f %f L %f %f ', xx0, yy0, xx, yy);
				endif
			endif
		endfor
		fprintf(out, '" fill="none" stroke="%s" stroke-width="1"/>\n', plotData{n}.color );
		endif
	endfor
	
	% Plots points
	for n=2:length(plotData)
		if( length(plotData{n}.symbol)>0 )
		for nx=1:length(plotData{n}.x)			
			% Checks ploint inside plot 
			if ( plotData{n}.x(nx)<xmin || plotData{n}.x(nx)>xmax || plotData{n}.y(nx)<ymin || plotData{n}.y(nx)>ymax)
				continue;
			endif
			xx=x+border_left+(plotData{n}.x(nx)-xmin)*(width-border_left-border_right)/(xmax-xmin);
			yy=y+border_top+(height-border_botton-border_top)-(plotData{n}.y(nx)-ymin)*(height-border_botton-border_top)/(ymax-ymin);
			fprintf(out, '<g stroke="%s" fill="%s"  transform="translate(%f,%f)  scale(%f)" stroke-width="0.2"  >\n', plotData{n}.symbolColor, plotData{n}.symbolColor, xx, yy,  plotData{n}.symbolScale);
			fprintf(out, '%s\n</g>\n', plotData{n}.symbol);
		endfor
		endif
	endfor

endfunction

% Returns first cuts with plot margins
function [x0,y0, ok]=intersec1(x0, y0, x1, y1, xmin, ymin, xmax, ymax)
	[x0,y0,x1,y1, ok]=intersect_n(x0, y0, x1, y1, xmin, ymin, xmax, ymax, 1);
	if(ok!=1)
		ok=0;
	endif
endfunction


% Returns cuts with plot margins
% if ok==1, line cuts in two points
function [x0,y0,x1,y1, ok]=intersec2(x0, y0, x1, y1, xmin, ymin, xmax, ymax)
	[x0,y0,x1,y1, ok]=intersect_n(x0, y0, x1, y1, xmin, ymin, xmax, ymax, 2);
	if(ok==2)
		ok=1;
	else
		ok=0;
	endif
endfunction

% Returns cuts withplot margins
% ok is number of cuts with margins
function [x0,y0,x1,y1, ok]=intersect_n(x0, y0, x1, y1, xmin, ymin, xmax, ymax, ncuts)
	ok=0; xx0=yy0=xx1=yy1=0;
	b=[
		xmin, ymin;
		xmax, ymin;
		xmax, ymax;
		xmin, ymax;
		xmin, ymin;
	];
	for n=2:length(b)
		[x,y, inside_ok]=line_cuts(x0, y0, x1, y1, b(n-1,1), b(n-1,2), b(n,1), b(n,2) );
		if(inside_ok==1)
			if(ok==0)
				xx0=x; yy0=y;
			else
				xx1=x; yy1=y;
			endif
			ok=ok+1;
			if(ok==ncuts)
				break;
			endif
		endif
	endfor
	x0=xx0; y0=yy0;
	x1=xx1; y1=yy1;
	if(ok==2)
		if(x0>x1)
			[x0,x1]=swap(x0,x1); [y0,y1]=swap(y0,y1);
		endif
	endif
endfunction

% Returns cut between segments (x0,y0)-(x1,y1) and (xmin, ymin)-(xmax, ymax)
% If inside_ok=-1, they don't cut.
% If inside_ok=0, (x0,y0)-(x1,y1) extension cuts to (xmin, ymin)-(xmax, ymax)
% If inside_ok=1, they cut
function [x,y, inside_ok]=line_cuts(x0, y0, x1, y1, xmin, ymin, xmax, ymax)
	%x0
	%y0
	%x1
	%y1
	inside_ok=-1; x=y=0;
	v=[x1-x0; y1-y0];
	vl=[xmax-xmin; ymax-ymin];
	M=[v, -vl];
	p=[xmin-x0; ymin-y0];
	if( det(M)==0 )
		return;
	endif
	
	param=M \ p;
	
	t=param(1);
	s=param(2);
	
	if( !between( 0, 1, s) )
		return;
	endif
	
	M=v*t + [ x0; y0 ];
	x=M(1); y=M(2);
	
	inside_ok=between(0,1,t);
endfunction

function [ok]=between(x0, x1, val)
	if(x0>x1)
		[x0,x1]=swap(x0,x1);
	endif
	
	ok=0;
	if(x0<=val && val<=x1)
		ok=1;
	endif
endfunction